"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Cryptrack = void 0;
const elliptic_1 = require("elliptic");
const crypto_js_1 = require("crypto-js");
const jose_1 = require("jose");
class Cryptrack {
    constructor() {
        this._curve = new elliptic_1.ec('secp256k1');
        this._keys = this._curve.genKeyPair();
        this._store = new Map();
    }
    getPublicKey() {
        return this._keys.getPublic('hex');
    }
    register(id, publicKeyHex) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const peerPubKey = this._curve.keyFromPublic(publicKeyHex, 'hex').getPublic();
                const sharedSecret = this._keys.derive(peerPubKey).toString(16);
                const sharedKey = (0, crypto_js_1.SHA256)(sharedSecret).toString();
                const jwk = {
                    kty: 'oct',
                    k: Buffer.from(sharedKey, 'hex').toString('base64url'),
                    alg: 'A256GCM',
                    ext: true,
                };
                const key = yield (0, jose_1.importJWK)(jwk, 'A256GCM');
                this._store.set(id, key);
            }
            catch (err) {
                throw new Error(`Invalid public key: ${err.message}`);
            }
        });
    }
    getKey(id) {
        const key = this._store.get(id);
        if (!key)
            throw new Error(`Shared key for ID "${id}" not found`);
        return key;
    }
    encrypt(id, plaintext) {
        return __awaiter(this, void 0, void 0, function* () {
            const key = this.getKey(id);
            const encoded = new TextEncoder().encode(plaintext);
            return yield new jose_1.CompactEncrypt(encoded)
                .setProtectedHeader({ alg: 'dir', enc: 'A256GCM' })
                .encrypt(key);
        });
    }
    decrypt(id, encrypted) {
        return __awaiter(this, void 0, void 0, function* () {
            const key = this.getKey(id);
            const { plaintext } = yield (0, jose_1.compactDecrypt)(encrypted, key);
            return new TextDecoder().decode(plaintext);
        });
    }
    signToken(id_1, payload_1) {
        return __awaiter(this, arguments, void 0, function* (id, payload, expires = "1h") {
            const sharedKey = this.getKey(id);
            return yield new jose_1.SignJWT(Object.assign({}, payload))
                .setProtectedHeader({ alg: 'HS256' })
                .setIssuedAt()
                .setExpirationTime(expires)
                .setSubject(id)
                .sign(sharedKey);
        });
    }
    verifyToken(token) {
        return __awaiter(this, void 0, void 0, function* () {
            const payload = (0, jose_1.decodeJwt)(token);
            const id = payload.sub;
            if (!id)
                throw new Error('Token missing subject');
            const sharedKey = this.getKey(id);
            const { payload: verifiedPayload } = yield (0, jose_1.jwtVerify)(token, sharedKey);
            return {
                id,
                payload: verifiedPayload,
            };
        });
    }
}
exports.Cryptrack = Cryptrack;
