import { ec } from 'elliptic';
import { SHA256 } from 'crypto-js';
import {
    CompactEncrypt,
    compactDecrypt,
    importJWK,
    decodeJwt,
    JWTPayload,
    SignJWT,
    jwtVerify
} from 'jose';

export class Cryptrack {
    private readonly _curve: ec;
    private readonly _keys: ec.KeyPair;
    private readonly _store: Map<string, CryptoKey>;

    constructor() {
        this._curve = new ec('secp256k1');
        this._keys = this._curve.genKeyPair();
        this._store = new Map();
    }

    getPublicKey(): string {
        return this._keys.getPublic('hex');
    }

    async register(id: string, publicKeyHex: string): Promise<void> {
        try {
            const peerPubKey = this._curve.keyFromPublic(publicKeyHex, 'hex').getPublic();
            const sharedSecret = this._keys.derive(peerPubKey).toString(16);
            const sharedKey = SHA256(sharedSecret).toString();

            const jwk = {
                kty: 'oct',
                k: Buffer.from(sharedKey, 'hex').toString('base64url'),
                alg: 'A256GCM',
                ext: true,
            };

            const key = await importJWK(jwk, 'A256GCM');
            this._store.set(id, key as CryptoKey);
        } catch (err) {
            throw new Error(`Invalid public key: ${(err as Error).message}`);
        }
    }

    getKey(id: string): CryptoKey {
        const key = this._store.get(id);
        if (!key) throw new Error(`Shared key for ID "${id}" not found`);
        return key;
    }

    async encrypt(id: string, plaintext: string): Promise<string> {
        const key = this.getKey(id);
        const encoded = new TextEncoder().encode(plaintext);

        return await new CompactEncrypt(encoded)
            .setProtectedHeader({ alg: 'dir', enc: 'A256GCM' })
            .encrypt(key);
    }

    async decrypt(id: string, encrypted: string): Promise<string> {
        const key = this.getKey(id);
        const { plaintext } = await compactDecrypt(encrypted, key);
        return new TextDecoder().decode(plaintext);
    }

    async signToken(id: string, payload: Record<string, any>, expires: string = "1h"): Promise<string> {
        const sharedKey = this.getKey(id);

        return await new SignJWT({ ...payload })
            .setProtectedHeader({ alg: 'HS256' })
            .setIssuedAt()
            .setExpirationTime(expires)
            .setSubject(id)
            .sign(sharedKey);
    }

    async verifyToken(token: string): Promise<{ id: string, payload: JWTPayload }> {
        const payload = decodeJwt(token);
        const id = payload.sub;

        if (!id) throw new Error('Token missing subject');

        const sharedKey = this.getKey(id);
        const { payload: verifiedPayload } = await jwtVerify(token, sharedKey);

        return {
            id,
            payload: verifiedPayload,
        };
    }
}
